/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    normalizeFields

Description

\*---------------------------------------------------------------------------*/

#include "OFstream.H"
#include "fvCFD.H"


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "addTimeOptions.H"
#   include "setRootCase.H"

#   include "createTime.H"

    // Get times list
    instantList Times = runTime.times();

    // set startTime and endTime depending on -time and -latestTime options
#   include "checkTimeOptions.H"

    runTime.setTime(Times[startTime], startTime);

#   include "createMesh.H"

    runTime.setTime(Times[endTime-1], endTime-1);

    // Read control dictionary
    IOdictionary sampleDict
    (
        IOobject
        (
            "normalizeFieldsDict",
            runTime.system(),
            mesh,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    );

    const wordList fieldNames = sampleDict.lookup("fields");

    for (label i=startTime; i<endTime; i++)
    {
        runTime.setTime(Times[i], i);

        Info<< "Time = " << runTime.timeName() << endl;
        
        forAll(fieldNames, fieldI)
        {
            const word& fieldName = fieldNames[fieldI];
                    
            IOobject fieldHeader
                (
                    fieldName,
                    runTime.timeName(),
                    mesh,
                    IOobject::MUST_READ
                );
            
            // Check field exists
            if (fieldHeader.headerOk())
            {
                mesh.readUpdate();
                
                volScalarField field(fieldHeader,mesh);

                dimensionedScalar favg = field.weightedAverage(mesh.V());
                volScalarField sqrdev=sqr(field - favg);
                dimensionedScalar fstd = sqrt(sqrdev.weightedAverage(sqr(mesh.V())));

                Info << "field: " << fieldName
                    << " mean: " << favg.value()
                    << " std: " << fstd.value()
                    << endl;

                // normalize field
                volScalarField nField
                    (
                        IOobject
                        (
                            "N"+fieldName,
                            runTime.timeName(),
                            mesh,
                            IOobject::NO_READ,
                            IOobject::AUTO_WRITE
                        ),
                        (field-favg)/fstd
                    );

                nField.write();
            }
            else
            {
                Info<< "    No " << fieldName << " or not scalar field " << endl;
            }
        }
    }
    
    return(0);
}


// ************************************************************************* //
