/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    passiveScalarTransport

Description
    Solves a transport equation for a passive scalar

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "incompressible/singlePhaseTransportModel/singlePhaseTransportModel.H"
#if WM_PROJECT_VERSION_MAJOR == 1 && WM_PROJECT_VERSION_MINOR < 5
#include "incompressible/turbulenceModel/turbulenceModel.H"
#else
#include "RAS/incompressible/RASModel/RASModel.H"
typedef incompressible::RASModel turbulenceModel;
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
    argList::validOptions.insert("reverse", "");

#   include "addTimeOptions.H"
#   include "setRootCase.H"

    bool reverseVelocity(args.options().found("reverse"));

#   include "createTime.H"

    // Get times list
    instantList Times = runTime.times();

    // set startTime and endTime depending on -time and -latestTime options
#   include "checkTimeOptions.H"

    runTime.setTime(Times[startTime], startTime);

#   include "createMesh.H"

    Info<< "\nCalculating passive scalar transport\n" << endl;

    for (label i=startTime; i<endTime; i++)
    {
        runTime.setTime(Times[i], i);

        Info<< nl << "Time = " << runTime.timeName() << nl << endl;

#   include "createFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#   include "CourantNo.H"

#   include "readSIMPLEControls.H"

        if (reverseVelocity)
        {
            phi=-phi;
        }

        for (int nonOrth=0; nonOrth<=nNonOrthCorr; nonOrth++)
        {
            solve
            (
                fvm::ddt(psi)
                + fvm::div(phi, psi)
                - fvm::laplacian(turbulence->nuEff(), psi)
                ==
                Spsi
            );
        }

        psi.write();
    }

    Info<< "End\n" << endl;

    return(0);
}


// ************************************************************************* //
